from pathlib import Path

from qbpm import profiles
from qbpm.profiles import Profile


def check_is_empty(path: Path):
    assert len(list(path.iterdir())) == 0


def check_empty_profile(profile: Profile | None):
    assert profile
    config_dir = profile.root / "config"
    assert list(profile.root.iterdir()) == [config_dir]
    assert list(config_dir.iterdir()) == []


def check_new_profile(profile: Profile):
    assert profile
    config_dir = profile.root / "config"
    assert list(profile.root.iterdir()) == [config_dir]
    assert list(config_dir.iterdir()) == [config_dir / "config.py"]


def test_set_profile(tmp_path: Path):
    assert Profile("test", tmp_path).root == tmp_path / "test"


def test_create_profile(tmp_path: Path):
    profile = Profile("test", tmp_path)
    assert profiles.create_profile(profile)
    assert list(tmp_path.iterdir()) == [profile.root]
    check_empty_profile(profile)


def test_create_profile_conflict(tmp_path: Path):
    (tmp_path / "test").touch()
    profile = Profile("test", tmp_path)
    assert not profiles.create_profile(profile)


def test_create_profile_parent(tmp_path: Path):
    profile = Profile("../test", tmp_path / "profiles")
    assert not profiles.create_profile(profile)
    assert not (tmp_path / "test").exists()


def test_create_profile_nested_conflict(tmp_path: Path):
    assert profiles.create_profile(Profile("test", tmp_path))
    assert not profiles.create_profile(Profile("test/a", tmp_path))


def test_create_config(tmp_path: Path):
    profile = Profile("test", tmp_path)
    config_dir = profile.root / "config"
    config_dir.mkdir(parents=True)
    profiles.create_config(profile, tmp_path)
    assert list(config_dir.iterdir()) == [config_dir / "config.py"]


def test_overwrite_config(tmp_path: Path):
    profile = Profile("test", tmp_path)
    url = "http://example.com"
    config_dir = profile.root / "config"
    config_dir.mkdir(parents=True)
    profiles.create_config(profile, tmp_path)
    profiles.create_config(profile, tmp_path, url, True)
    assert list(config_dir.iterdir()) == [config_dir / "config.py"]
    with (config_dir / "config.py").open() as conf:
        for line in conf:
            if url in line:
                return
    raise AssertionError()


def test_new_profile(tmp_path: Path):
    (tmp_path / "config.py").touch()
    profile = Profile("test", tmp_path / "test")
    assert profiles.new_profile(profile, tmp_path, desktop_file=False)
    check_new_profile(profile)



import platform
import sys
from collections.abc import Iterator
from dataclasses import dataclass, replace
from os import environ
from pathlib import Path
from shutil import which

from .log import error, or_phrase


@dataclass
class Dmenu:
    name: str
    args: str

    def installed(self) -> bool:
        return which(self.name) is not None

    def command(self, prompt: str, qb_args: str) -> str:
        prompt = prompt or "''"
        return f"{self.name} {self.args.format(prompt=prompt, qb_args=qb_args)}"

    def commandline(self, profiles: list[str], prompt: str, qb_args: str) -> str:
        profile_list = "\n".join(profiles)
        return f'echo "{profile_list}" | {self.command(prompt, qb_args)}'


class ApplescriptMenu:
    @classmethod
    def installed(cls) -> bool:
        return platform.system() == "Darwin"

    @classmethod
    def commandline(cls, profiles: list[str], _prompt: str, qb_args: str) -> str:
        profile_list = '", "'.join(profiles)
        return f"""osascript -e \'set profiles to {{"{profile_list}"}}
set profile to choose from list profiles with prompt "qutebrowser: {qb_args}" default items {{item 1 of profiles}}
item 1 of profile\'"""


def find_menu(menu: str | None) -> Dmenu | ApplescriptMenu | None:
    menus = list(supported_menus())
    if not menu:
        found = next(filter(lambda m: m.installed(), menus), None)
        if not found:
            error(
                "no menu program found, use --menu to provide a dmenu-compatible menu or install one of "
                + or_phrase([m.name for m in menus if isinstance(m, Dmenu)])
            )
        return found
    dmenu = custom_dmenu(menu)
    if not dmenu.installed():
        error(f"{dmenu.name} not found")
        return None
    return dmenu


def custom_dmenu(command: str) -> Dmenu:
    split = command.split(" ", maxsplit=1)
    if len(split) == 1 or not split[1]:
        name = Path(command).name
        for menu in supported_menus():
            if isinstance(menu, Dmenu) and menu.name == name:
                return menu if menu.name == command else replace(menu, name=command)
    return Dmenu(split[0], split[1] if len(split) == 2 else "")


def supported_menus() -> Iterator[Dmenu | ApplescriptMenu]:
    if ApplescriptMenu.installed():
        yield ApplescriptMenu()
    if environ.get("WAYLAND_DISPLAY"):
        yield from [
            # default window is too narrow for a long prompt
            Dmenu("fuzzel", "--dmenu"),
            Dmenu("wofi", "--dmenu -p {prompt}"),
            Dmenu("dmenu-wl", "-p {prompt}"),
        ]
    if environ.get("DISPLAY"):
        yield from [
            Dmenu(
                "rofi",
                "-dmenu -no-custom -p {prompt} -mesg '{qb_args}'",
            ),
            Dmenu("dmenu", "-p {prompt}"),
        ]
    if sys.stdin.isatty():
        if environ.get("TMUX"):
            yield Dmenu("fzf-tmux", "--prompt '{prompt}> '")
        yield Dmenu("fzf", "--prompt '{prompt}> '")



from os import chdir, environ
from pathlib import Path

from click.testing import CliRunner

from qbpm.main import main

no_desktop = "--no-desktop-file"


def test_profile_dir_option(tmp_path: Path):
    (tmp_path / "config.py").touch()
    runner = CliRunner()
    result = runner.invoke(
        main, ["-P", str(tmp_path), "new", "-C", str(tmp_path), no_desktop, "test"]
    )
    assert result.exit_code == 0
    assert result.output.strip() == str(tmp_path / "test")
    assert tmp_path / "test" in list(tmp_path.iterdir())


def test_profile_dir_env(tmp_path: Path):
    environ["QBPM_PROFILE_DIR"] = str(tmp_path)
    (tmp_path / "config.py").touch()
    runner = CliRunner()
    result = runner.invoke(main, ["new", "-C", str(tmp_path), no_desktop, "test"])
    assert result.exit_code == 0
    assert result.output.strip() == str(tmp_path / "test")
    assert tmp_path / "test" in list(tmp_path.iterdir())


def test_config_dir_option(tmp_path: Path):
    environ["QBPM_PROFILE_DIR"] = str(tmp_path)
    config = tmp_path / "config.py"
    config.touch()
    runner = CliRunner()
    result = runner.invoke(main, ["new", "-C", str(tmp_path), no_desktop, "test"])
    assert result.exit_code == 0
    assert str(config) in (tmp_path / "test/config/config.py").read_text()


def test_relative_config_dir(tmp_path: Path):
    environ["QBPM_PROFILE_DIR"] = str(tmp_path)
    config = tmp_path / "config.py"
    config.touch()
    chdir(tmp_path)
    runner = CliRunner()
    result = runner.invoke(main, ["new", "-C", ".", no_desktop, "test"])
    assert result.exit_code == 0
    assert str(config) in (tmp_path / "test/config/config.py").read_text()


def test_from_session(tmp_path: Path):
    environ["QBPM_PROFILE_DIR"] = str(tmp_path)
    (tmp_path / "config.py").touch()
    session = tmp_path / "test.yml"
    session.write_text("windows:\n")
    runner = CliRunner()
    result = runner.invoke(
        main, ["from-session", "-C", str(tmp_path), no_desktop, str(session)]
    )
    assert result.exit_code == 0
    assert result.output.strip() == str(tmp_path / "test")
    assert (tmp_path / "test/data/sessions/_autosave.yml").read_text() == ("windows:\n")



from pathlib import Path

from qbpm import Profile
from qbpm.desktop import create_desktop_file

TEST_DIR = Path(__file__).resolve().parent


def test_create_desktop_file(tmp_path: Path):
    application_path = tmp_path / "applications"
    application_path.mkdir()
    profile = Profile("test", tmp_path)
    create_desktop_file(profile, application_path)
    assert (application_path / "test.desktop").read_text() == (
        TEST_DIR / "test.desktop"
    ).read_text().replace("{qbpm}", " ".join(profile.cmdline()))



from os import environ
from pathlib import Path

from qbpm.choose import choose_profile, find_menu


def write_script(parent_dir: Path, name: str = "menu", contents: str = "") -> Path:
    parent_dir.mkdir(exist_ok=True)
    menu = parent_dir / name
    menu.write_text(f"#!/bin/sh\n{contents}")
    menu.chmod(0o700)
    return menu


def test_choose(tmp_path: Path):
    log = tmp_path / "log"
    log.touch()
    menu = write_script(tmp_path / "bin", contents=f"cat > {log}\necho p1")
    write_script(
        tmp_path / "bin",
        name="qutebrowser",
        contents=f'echo qutebrowser "$@" >> {log}',
    )
    environ["PATH"] = str(tmp_path / "bin") + ":" + environ["PATH"]

    profile_dir = tmp_path / "profiles"
    profile_dir.mkdir()
    (profile_dir / "p1").mkdir()
    (profile_dir / "p2").mkdir()
    assert choose_profile(profile_dir, str(menu), False, ())
    assert log.read_text().startswith(
        f"""p1
p2
qutebrowser
qutebrowser -B {profile_dir / "p1"}"""
    )


def test_find_installed_menu(tmp_path: Path):
    write_script(tmp_path / "bin", name="dmenu")
    environ["PATH"] = str(tmp_path / "bin")
    environ["DISPLAY"] = ":1"
    assert getattr(find_menu(None), "name", None) == "dmenu"


def test_override_menu_priority(tmp_path: Path):
    write_script(tmp_path / "bin", name="fuzzel")
    write_script(tmp_path / "bin", name="dmenu-wl")
    environ["PATH"] = str(tmp_path / "bin")
    environ["WAYLAND_DISPLAY"] = "wayland-2"
    assert getattr(find_menu(None), "name", None) == "fuzzel"
    assert getattr(find_menu("dmenu-wl"), "name", None) == "dmenu-wl"


def test_custom_menu():
    dmenu = find_menu("/bin/sh -c")
    assert dmenu is not None
    assert (
        dmenu.commandline(["p1", "p2"], "", "").strip() == 'echo "p1\np2" | /bin/sh -c'
    )


def test_invalid_custom_menu():
    assert find_menu("fake_command") is None


def test_custom_menu_default_args(tmp_path: Path):
    menu = write_script(tmp_path / "bin", name="rofi")
    environ["PATH"] = str(tmp_path / "bin")
    environ["DISPLAY"] = ":1"
    dmenu = find_menu(str(menu))
    assert dmenu is not None
    assert f"{menu} -dmenu -no-custom -p ''" in dmenu.commandline(["p1", "p2"], "", "")


def test_custom_menu_custom_args(tmp_path: Path):
    menu = write_script(tmp_path / "bin", name="rofi")
    command = f"{menu} -custom -dmenu"
    environ["PATH"] = str(tmp_path / "bin")
    environ["DISPLAY"] = ":1"
    dmenu = find_menu(command)
    assert dmenu is not None
    assert dmenu.commandline(["p1", "p2"], "", "").endswith(command)
